package collections;
import java.util.Vector;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+Coloriage/attributs.java*/
public class Coloriage {
  private GrapheNonOriente graphe_;
  public Coloriage(GrapheNonOriente g) {
    graphe_ = g;
  }
/*-*/
/*+Coloriage/reinitialiserPossibles.java*/
  private void reinitialiserPossibles() {
    for(Curseur<Sommet> cs = graphe_.premierSommet(); cs != null;
        cs = graphe_.sommetSuivant(cs)) {
      Sommet s =  graphe_.sommet(cs);
      DonneesGC donnees = (DonneesGC)s.donneesAlgorithme;
      donnees.possible = donnees.couleur == -1;
    }
  }
/*-*/
/*+Coloriage/meilleurSommet.java*/
  private Sommet meilleurSommet() {
    int plusGrandNombre = -1;
    Sommet meilleur = null;
    for(Curseur<Sommet> cs = graphe_.premierSommet(); cs != null;
        cs = graphe_.sommetSuivant(cs)) {
      Sommet s =  graphe_.sommet(cs);
      DonneesGC donnees = (DonneesGC)s.donneesAlgorithme;
      if (donnees.possible && donnees.nombreDeVoisinsNonColories >
          plusGrandNombre) {
        meilleur = s;
        plusGrandNombre = donnees.nombreDeVoisinsNonColories;
      }
    }
    return meilleur;
  }
/*-*/
/*+Coloriage/actualiserSommets.java*/
  private void actualiserSommets(Sommet colorie, int couleur) {
    for(Curseur<Arete> ca = graphe_.premiereArete(colorie); ca != null;
        ca = graphe_.areteSuivante(ca)) {
      Arete a =  graphe_.arete(ca);
      Sommet s1 = a.sommet1();
      Sommet s2 = a.sommet2();
      Sommet voisin;
      if (s1 == colorie)
        voisin = s2;
      else
        voisin = s1;
      DonneesGC donnees = (DonneesGC)voisin.donneesAlgorithme;
      donnees.nombreDeVoisinsNonColories--;
      donnees.possible = false;
    }
  }
/*-*/
/*+Coloriage/colorier.java*/
  public void colorier() {
// Mise en place des donnees accroches  chaque sommet utiles au coloriage
    for(Curseur<Sommet> cs = graphe_.premierSommet(); cs != null;
        cs = graphe_.sommetSuivant(cs)) {
      Sommet s = graphe_.sommet(cs);
      DonneesGC donnees = new DonneesGC();
      donnees.nombreDeVoisinsNonColories = graphe_.degre(s);
      s.donneesAlgorithme = donnees;
    }
    int nombreDeSommetsNonColories = graphe_.nombreDeSommets();
    for(int couleur = 1;;couleur++) {
      if (nombreDeSommetsNonColories == 0) break;
      reinitialiserPossibles();
      for(;;) {
        Sommet aColorier = meilleurSommet();
        if (aColorier == null) break;
        DonneesGC donnees = (DonneesGC)aColorier.donneesAlgorithme;
        donnees.possible = false;
        donnees.couleur = couleur;
        nombreDeSommetsNonColories--;
        actualiserSommets(aColorier, couleur);
      }
    }
  }
/*-*/
  public static void test(GrapheNonOriente g) throws java.io.FileNotFoundException, java.io.IOException {
    Coloriage col = new Coloriage(g);
    col.colorier();
    for(Curseur<Sommet> cs = g.premierSommet(); cs != null; cs = g.sommetSuivant(cs)) {
      Sommet s = g.sommet(cs);
      DonneesGC donneesA = (DonneesGC)s.donneesAlgorithme;
      DonneesSommet donneesU = (DonneesSommet)s.donneesUtilisateur;
      System.out.println("Sommet "+donneesU+" : couleur "+donneesA.couleur);
    }
  }

  public static void main(String[] args) throws java.io.FileNotFoundException, java.io.IOException{
    GrapheNonOriente g1 = new GrapheNonOrienteMatrice();
    GrapheNonOriente g2 = new GrapheNonOrienteListeChainee();
    g1.lire("C:\\Mes documents\\g4.txt");
    g2.lire("C:\\Mes documents\\g4.txt");
    test(g1);
    System.out.println();
    test(g2);
  }
/*+Coloriage/DonneesGC.java*/
  private class DonneesGC {
    public int couleur;
    public int nombreDeVoisinsNonColories;
    public boolean possible;

    public DonneesGC() {
      couleur = -1;
      nombreDeVoisinsNonColories = 0;
      possible= true;
    }
  }
/*-*/

}