package collections;
import java.util.Vector;
import java.io.*;
import java.util.HashMap;
import java.util.StringTokenizer;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+Dijkstra/Dijkstra.java*/
public class Dijkstra {
  private GrapheOriente graphe_;

  public Dijkstra(GrapheOriente g) {
    graphe_ = g;
  }
/*-*/
/*+Dijkstra/plusCourtChemin.java*/
  public ListeRec<Arc> plusCourtChemin(Sommet depart, Sommet arrivee) {
    return plusCourtCheminGeneral(depart, arrivee, false);
  }
/*-*/
/*+Dijkstra/plusCourtCheminEnArcs.java*/
  public ListeRec<Arc> plusCourtCheminEnArcs(Sommet depart, Sommet arrivee) {
    return plusCourtCheminGeneral(depart, arrivee, true);
  }
/*-*/
/*+Dijkstra/sommetDistanceMinimale.java*/
  private Sommet sommetDistanceMinimale() {
    int distanceMini = Integer.MAX_VALUE;
    Sommet sommetMini = null;
    for (Curseur<Sommet> c = graphe_.premierSommet(); 
         c!= null; c = graphe_.sommetSuivant(c)) {
      Sommet s = graphe_.sommet(c);
      InfosSommet infos= (InfosSommet)s.donneesAlgorithme;
      if (!infos.minTrouve && infos.distance < distanceMini) {
        distanceMini = infos.distance;
        sommetMini = s;
      }
    }
    return sommetMini;
  }
/*-*/
/*+Dijkstra/actualiserDistances.java*/
  private void actualiserDistances(Sommet sommetMini, boolean enArcs) {
    InfosSommet infosSommetMini= (InfosSommet)sommetMini.donneesAlgorithme;
    for (Curseur<Arc> ca = graphe_.premierArc(sommetMini); 
         ca != null; ca = graphe_.arcSuivant(ca)) {
      Arc a = graphe_.arc(ca);
      Sommet successeur = a.arrivee();
      InfosSommet infosSuccesseur = (InfosSommet)successeur.donneesAlgorithme;
      int valeur = 1;
      if (!enArcs)
        valeur = a.cout().intValue();
      if (infosSuccesseur.distance > (infosSommetMini.distance + valeur)) {
        infosSuccesseur.distance = infosSommetMini.distance + valeur;
        infosSuccesseur.pere = a;
      }
    }
  }
/*-*/
/*+Dijkstra/plusCourtCheminGeneral.java*/
  private ListeRec<Arc> plusCourtCheminGeneral(Sommet depart, 
                                               Sommet arrivee,
                                               boolean enArcs) {
// Initialisation des donnes associes  chaque sommet
    if (depart == arrivee) 
      return new ListeRecChainee<Arc>();
    for (Curseur<Sommet> c = graphe_.premierSommet(); c!= null; 
         c = graphe_.sommetSuivant(c)) {
      InfosSommet infos = new InfosSommet();
      if (graphe_.sommet(c) == depart) {
        infos.distance = 0;
      }
      graphe_.sommet(c).donneesAlgorithme = infos;
    }
    for(;;) {
      // Recherche du sommet ayant la plus petite distance
      Sommet sommetMini = sommetDistanceMinimale();
      if (sommetMini == null)
        break;
      if (sommetMini == arrivee)
        break;
      InfosSommet infosSommetMini=
          (InfosSommet)sommetMini.donneesAlgorithme;
      infosSommetMini.minTrouve = true;
      actualiserDistances(sommetMini, enArcs);
    }
    // Construction du chemin
    ListeRec<Arc> chemin = new ListeRecChainee<Arc>();
    for(Sommet s = arrivee;;) {
      Arc duPere = ((InfosSommet)s.donneesAlgorithme).pere;
      if(duPere == null) break;
      chemin.ajouterDebut(duPere);
      s = duPere.depart();
    }
    if (chemin.estvide() || chemin.premier().depart() != depart)
      chemin = null;
    return chemin;
  }
/*-*/
  public static void testPlusCourtChemin(GrapheOriente g) 
         throws java.io.FileNotFoundException, java.io.IOException {
    HashMap<String, Sommet> sommets;

    sommets = g.lire("C:\\Mes documents\\g2.txt");
    Dijkstra d = new Dijkstra(g);
    ListeRec<Arc> chemin = d.plusCourtChemin(sommets.get("1"), 
                                             sommets.get("22"));
    for (Curseur<Arc> c = chemin.debut(); c != null; c = chemin.suivant(c)) {
      System.out.print(chemin.element(c).donneesUtilisateur + " ");
    }
    System.out.println();
    
    chemin = d.plusCourtCheminEnArcs(sommets.get("1"),
                                     sommets.get("22"));
    for (Curseur<Arc> c = chemin.debut(); c != null; c = chemin.suivant(c)) {
      System.out.print(chemin.element(c).donneesUtilisateur + " ");
    }
    System.out.println();
  }

  public static void main(String[] args) 
         throws java.io.FileNotFoundException, java.io.IOException {
    GrapheOriente g1 = new GrapheOrienteMatrice();
    GrapheOriente g2 = new GrapheOrienteListeChainee();

    testPlusCourtChemin(g1);
    testPlusCourtChemin(g2);
  }

/*+Dijkstra/InfosSommet.java*/
  private class InfosSommet {
    public boolean minTrouve;
    public int distance;
    public Arc pere;

    public InfosSommet() {
      pere = null;
      distance = Integer.MAX_VALUE;
      minTrouve = false;
    }
  }
/*-*/
}