package collections;
import java.util.Vector;
import java.util.HashMap;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+Ford/attributs.java*/
public class Ford {
  private GrapheOriente graphe_;
  private GrapheOriente grapheEcart_;
  private Sommet entreeEcart_, sortieEcart_;

  public Ford(GrapheOriente g) {
    graphe_ = g;
  }
/*-*/
/*+Ford/construireGrapheEcart.java*/
  private void construireGrapheEcart(Sommet entree, Sommet sortie) {
    grapheEcart_ = new GrapheOrienteListeChainee();
    for(Curseur<Sommet> c = graphe_.premierSommet(); 
        c != null; c = graphe_.sommetSuivant(c)) {
      Sommet s = graphe_.sommet(c);
      Sommet se = grapheEcart_.ajouterSommet();
      if (s == entree) entreeEcart_ = se;
      if (s == sortie) sortieEcart_ = se;
      se.donneesUtilisateur = s;
      s.donneesAlgorithme = se;
    }
    for(Curseur<Sommet> c = graphe_.premierSommet(); 
        c != null; c = graphe_.sommetSuivant(c)) {
      Sommet s = graphe_.sommet(c);
      for (Curseur<Arc> ca = graphe_.premierArc(s); 
           ca != null; ca = graphe_.arcSuivant(ca)) {
        Arc a = graphe_.arc(ca);
        Sommet s1 = a.depart();
        Sommet s2 = a.arrivee();
        Arc ae = grapheEcart_.ajouterArc((Sommet)s1.donneesAlgorithme,
                                         (Sommet)s2.donneesAlgorithme,
                                         a.cout());
        DonneesArc da = new DonneesArc(a);
        ae.donneesUtilisateur = da;
      }
    }
  }

/*-*/
/*+Ford/plusCourtCheminEnArcs.java*/
  private ListeRec<Arc> plusCourtCheminEnArcs() {
    Dijkstra d = new Dijkstra(grapheEcart_);
    return d.plusCourtCheminEnArcs(entreeEcart_, sortieEcart_);
  }
/*-*/
/*+Ford/plusPetitArc.java*/
  private int plusPetitArc(ListeRec<Arc> chemin) {
    int coutMin = Integer.MAX_VALUE;
    for(Curseur<Arc> c = chemin.debut(); c != null ; c = chemin.suivant(c)) {
      Arc a = chemin.element(c);
      int cout = a.cout().intValue();
      if (cout < coutMin) {
        coutMin = cout;
      }
    }
    return coutMin;
  }
/*-*/
/*+Ford/actualiserGrapheEcart.java*/
  private void actualiserGrapheEcart(ListeRec<Arc> chemin) {
    int coutMin = plusPetitArc(chemin);
    for (Curseur<Arc> c = chemin.debut(); c != null ; c = chemin.suivant(c)) {
      Arc ad = chemin.element(c);
      int coutd = ad.cout().intValue();
      Sommet s1 = ad.depart();
      Sommet s2 = ad.arrivee();
      Arc ai = ((DonneesArc)ad.donneesUtilisateur).arcInverse;
      Arc ao = ((DonneesArc)ad.donneesUtilisateur).arcOriginal;
      boolean arcFlot = ((DonneesArc)ad.donneesUtilisateur).arcFlot;
      if (ai == null) {
        ai = grapheEcart_.ajouterArc(s2, s1, coutMin);
        ai.donneesUtilisateur = new DonneesArc(ao, ad, !arcFlot);
        ((DonneesArc)ad.donneesUtilisateur).arcInverse = ai;
      } else {
        grapheEcart_.valuerArc(ai, ai.cout().intValue() + coutMin);
      }
      if (ad.cout().intValue() == coutMin) {
        grapheEcart_.supprimerArc(ad);
        ((DonneesArc)ai.donneesUtilisateur).arcInverse = null;
      } else {
        grapheEcart_.valuerArc(ad, ad.cout().intValue() - coutMin);
      }
    }
  }
/*-*/
/*+Ford/construireFlot.java*/
  private GrapheOriente construireFlot() {
    GrapheOriente flot;
    flot = new GrapheOrienteListeChainee();
    for(Curseur<Sommet> c = grapheEcart_.premierSommet(); c != null;
        c = grapheEcart_.sommetSuivant(c)) {
      Sommet s = grapheEcart_.sommet(c);
      Sommet sf = flot.ajouterSommet();
      sf.donneesUtilisateur = s.donneesUtilisateur;
      s.donneesUtilisateur = sf;
    }
    for(Curseur<Sommet> c = grapheEcart_.premierSommet(); c != null;
        c = grapheEcart_.sommetSuivant(c)) {
      Sommet s = grapheEcart_.sommet(c);
      for (Curseur<Arc> ca = grapheEcart_.premierArc(s); ca != null;
            ca = grapheEcart_.arcSuivant(ca)) {
        Arc a = grapheEcart_.arc(ca);
        Sommet s1 = a.depart();
        Sommet s2 = a.arrivee();
        if (!((DonneesArc)a.donneesUtilisateur).arcFlot) continue;
        Arc af = flot.ajouterArc((Sommet)s2.donneesUtilisateur,
                                 (Sommet)s1.donneesUtilisateur, a.cout());
        af.donneesUtilisateur = a;
      }
    }
    for(Curseur<Sommet> c = grapheEcart_.premierSommet(); c != null;
        c = grapheEcart_.sommetSuivant(c)) {
      Sommet s = grapheEcart_.sommet(c);
      s.donneesUtilisateur = null;
    }
    return flot;
  }
/*-*/
/*+Ford/flotMax.java*/
  public GrapheOriente flotMax(Sommet entree, Sommet sortie)  {
    construireGrapheEcart(entree, sortie);
    for(;;) {
      ListeRec<Arc> chemin = plusCourtCheminEnArcs();
      if (chemin == null) 
        break;
      actualiserGrapheEcart(chemin);
    }
    return construireFlot();
  }
/*-*/
/*+Ford/DonneesArc.java*/
  private class DonneesArc {
    public Arc arcOriginal;
    public Arc arcInverse;
    public boolean arcFlot;
    public DonneesArc(Arc original) {
      arcOriginal = original;
      arcInverse = null;
      arcFlot = false;
    }

    public DonneesArc(Arc original, Arc inverse, boolean flot) {
      arcOriginal = original;
      arcInverse = inverse;
      arcFlot = flot;
    }
  }
/*-*/
/*+Ford/afficherFlot.java*/
  public static void afficherFlot(GrapheOriente g, GrapheOriente flot) {
    for(Curseur<Sommet> cs = flot.premierSommet(); 
        cs!=null; cs = flot.sommetSuivant(cs)) {
      Sommet s = flot.sommet(cs);
      for(Curseur<Arc> ca = flot.premierArc(s); 
          ca != null; ca = flot.arcSuivant(ca)) {
        Arc a = flot.arc(ca);
        Number valeur = a.cout();
        Sommet s1 = (Sommet)s.donneesUtilisateur;
        Sommet s2 = (Sommet)(a.arrivee()).donneesUtilisateur;
        System.out.println(s1.donneesUtilisateur+ ", " 
                         + s2.donneesUtilisateur + " : " + valeur);
      }
    }
  }
/*-*/
  public static void main(String[] args)
         throws java.io.FileNotFoundException, java.io.IOException {
    GrapheOrienteMatrice g = new GrapheOrienteMatrice();
    HashMap<String, Sommet> sommets;

    sommets = g.lire("C:\\Mes documents\\g3.txt");
    Ford f = new Ford(g);
    GrapheOriente flot = f.flotMax(sommets.get("1"),
                                   sommets.get("2"));
    afficherFlot(g,flot);
  }

}