package collections;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+GrapheNonOrienteListeChainee/attributs.java*/
public class GrapheNonOrienteListeChainee extends GrapheNonOriente {
  private ListeRec<Sommet> sommets_;
/*-*/
/*+GrapheNonOrienteListeChainee/constructeur.java*/
  public GrapheNonOrienteListeChainee() {
    sommets_ = new ListeRecChainee<Sommet>();
  }
/*-*/
/*+GrapheNonOrienteListeChainee/ajouterSommet.java*/
  public Sommet ajouterSommet() {
    Sommet s = new SommetGNOLC(this);
    sommets_.ajouterDebut(s);
    return s;
  }
/*-*/
/*+GrapheNonOrienteListeChainee/ajouterArete.java*/
  public Arete ajouterArete(Sommet sommet1, Sommet sommet2, Number cout) {
    SommetGNOLC sommet1p = (SommetGNOLC)sommet1;
    SommetGNOLC sommet2p = (SommetGNOLC)sommet2;

    prerequis(sommet1p.proprietaire_ == this);
    prerequis(sommet2p.proprietaire_ == this);
    AreteGNOLC arete = new AreteGNOLC(sommet1p, sommet2p, cout, this);
    sommet1p.successeurs.ajouterDebut(arete);
    sommet2p.successeurs.ajouterDebut(arete);
    return arete;
  }
/*-*/
/*+GrapheNonOrienteListeChainee/valuerArete.java*/
  public void valuerArete(Arete a, Number valeur) {
    AreteGNOLC ap = (AreteGNOLC)a;

    prerequis(ap.proprietaire_ == this);
    ap.cout_ = valeur;
  }
/*-*/
/*+GrapheNonOrienteListeChainee/supprimerSommet.java*/
  public void supprimerSommet(Sommet s) {
    SommetGNOLC sp = (SommetGNOLC)s;

    prerequis(sp.proprietaire_ == this);
    sommets_.enlever(s);
    for (Curseur<Sommet> cs = sommets_.debut();
         cs != null; cs = sommets_.suivant(cs)) {
      ListeRec<Arete> aretes = ((SommetGNOLC)sommets_.element(cs)).successeurs;
      Curseur<Arete> cour, prec;
      for (prec = null, cour = aretes.debut(); cour != null; ) {
        Arete a = aretes.element(cour);
        cour = aretes.suivant(cour);
        if (a.sommet1() == s || a.sommet2() == s) {
          if (prec == null) {
            aretes.enleverPremier();
          }
          else {
            aretes.supprimerSuivant(prec);
          }
        }
        else
          prec = cour;
      }
    }
  }
/*-*/
/*+GrapheNonOrienteListeChainee/supprimerArete.java*/
  public void supprimerArete(Arete a) {
    AreteGNOLC ap = (AreteGNOLC)a;
    SommetGNOLC sommet1 = (SommetGNOLC)a.sommet1();
    SommetGNOLC sommet2 = (SommetGNOLC)a.sommet2();

    prerequis(ap.proprietaire_ == this);
    ListeIter<ListeRec<Arete>> la = new ListeIterChainee<ListeRec<Arete>>();
    la.inserer(1, sommet1.successeurs);
    la.inserer(2, sommet2.successeurs);
    for (int i = 1; i <= 2; i++) {
      ListeRec<Arete> aretes = la.ieme(i);
      Curseur<Arete> cour, prec;
      for (prec = null, cour = aretes.debut(); cour != null; prec = cour,
           cour = aretes.suivant(cour)) {
        Arete arete = aretes.element(cour);
        if (arete == a) {
          if (prec == null) {
            aretes.enleverPremier();
          }
          else {
            aretes.supprimerSuivant(prec);
          }
          break;
        }
      }
    }
  }
/*-*/
/*+GrapheNonOrienteListeChainee/areteExiste.java*/
  public boolean areteExiste(Sommet sommet1, Sommet sommet2) {
    SommetGNOLC sommet1p = (SommetGNOLC)sommet1;
    SommetGNOLC sommet2p = (SommetGNOLC)sommet2;

    prerequis(sommet1p.proprietaire_ == this);
    prerequis(sommet2p.proprietaire_ == this);
    ListeRec<Arete> aretes = sommet1p.successeurs;
    for (Curseur<Arete> c = aretes.debut(); c != null; c = aretes.suivant(c)) {
      Arete a = aretes.element(c);
      if (a.sommet2() == sommet2) return true;
    }
    return false;
  }
/*-*/
/*+GrapheNonOrienteListeChainee/degre.java*/
  public int degre(Sommet s) {
    SommetGNOLC sp = (SommetGNOLC)s;

    prerequis(sp.proprietaire_ == this);
    return sp.successeurs.longueur();
  }
/*-*/
/*+GrapheNonOrienteListeChainee/nombreDeSommets.java*/
  public int nombreDeSommets() {
    return sommets_.longueur();
  }
/*-*/
/*+GrapheNonOrienteListeChainee/premiereArete.java*/
  public Curseur<Arete> premiereArete(Sommet s) {
    SommetGNOLC sp = (SommetGNOLC)s;
    prerequis(sp.proprietaire_ == this);
    if (sp.successeurs.debut() == null)
      return null;
    return new CurseurSuccesseur(sp, sp.successeurs.debut());
  }
/*-*/
/*+GrapheNonOrienteListeChainee/areteSuivante.java*/
  public Curseur<Arete> areteSuivante(Curseur<Arete> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    Curseur<Arete> suivant = cs.sommet1.successeurs.suivant(cs.arete);
    if (suivant == null)
      return null;
    return new CurseurSuccesseur(cs.sommet1, suivant);
  }
/*-*/
/*+GrapheNonOrienteListeChainee/arete.java*/
  public Arete arete(Curseur<Arete> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    return (Arete)cs.sommet1.successeurs.element(cs.arete);
  }
/*-*/
/*+GrapheNonOrienteListeChainee/premierSommet.java*/
  public Curseur<Sommet> premierSommet() {
    return sommets_.debut();
  }
/*-*/
/*+GrapheNonOrienteListeChainee/sommetSuivant.java*/
  public Curseur<Sommet> sommetSuivant(Curseur<Sommet> c) {
    return sommets_.suivant(c);
  }
/*-*/
/*+GrapheNonOrienteListeChainee/sommet.java*/
  public Sommet sommet(Curseur<Sommet> c) {
    return sommets_.element(c);
  }
/*-*/
/*+GrapheNonOrienteListeChainee/SommetGNOLC.java*/
  private class SommetGNOLC extends Sommet {
    public ListeRecChainee<Arete> successeurs;
    public GrapheNonOrienteListeChainee proprietaire_;

    public SommetGNOLC(GrapheNonOrienteListeChainee g) {
      successeurs = new ListeRecChainee<Arete>();
      proprietaire_ = g;
    }
  }
/*-*/
/*+GrapheNonOrienteListeChainee/AreteGNOLC.java*/
  private class AreteGNOLC extends Arete {
    public SommetGNOLC sommet1_;
    public SommetGNOLC sommet2_;
    public GrapheNonOrienteListeChainee proprietaire_;
    public Number cout_;

    public AreteGNOLC(SommetGNOLC sommet1, SommetGNOLC sommet2, 
                      Number cout, GrapheNonOrienteListeChainee g) {
      sommet1_ = sommet1;
      sommet2_ = sommet2;
      cout_ = cout;
      proprietaire_ = g;
    }

    public Sommet sommet1() {
      return sommet1_;
    }

    public Sommet sommet2() {
      return sommet2_;
    }

    public Number cout() {
      return cout_;
    }
  }
/*-*/
/*+GrapheNonOrienteListeChainee/CurseurSuccesseur.java*/
  private class CurseurSuccesseur extends Curseur<Arete> {
    public SommetGNOLC sommet1;
    public Curseur<Arete> arete;

    public CurseurSuccesseur(SommetGNOLC dep, Curseur<Arete> a) {
      sommet1 = dep;
      arete = a;
    }

  }
/*-*/
}
