package collections;
import java.util.Vector;
import java.util.Stack;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+GrapheNonOrienteMatrice/attributs.java*/
public class GrapheNonOrienteMatrice extends GrapheNonOriente {
  private Sommet sommets_[];
  private ListeRecChainee<Arete>[][] matrice_;
  private Stack<Integer> sommetsLibres_;
  private int dimensionMatrice_;
  private int nombreDeSommets_;
  private final int DELTA = 10;
/*-*/
  @SuppressWarnings("unchecked")
  private ListeRecChainee<Arete>[][] creerMatrice(int taille) {
    int[] dimensions = {taille, taille};
    return (ListeRecChainee<Arete>[][])
      java.lang.reflect.Array.newInstance(ListeRecChainee.class, dimensions);
  }

/*+GrapheNonOrienteMatrice/constructeur.java*/
  public GrapheNonOrienteMatrice() {
    matrice_ = creerMatrice(DELTA);
    sommets_ = new Sommet[DELTA];
    dimensionMatrice_ = 0;
    nombreDeSommets_ = 0;
    sommetsLibres_ = new Stack<Integer>();
  }
/*-*/
/*+GrapheNonOrienteMatrice/ajusterMatrice.java*/
    private void ajusterMatrice(int dimension) {
      if (matrice_.length < dimension) {
        int nd = matrice_.length + DELTA;
        ListeRecChainee<Arete>[][] nouvelle = creerMatrice(nd);
        Sommet[] nouveau = new Sommet[nd];
        for (int i = 0; i < dimensionMatrice_; i++)
          for (int j = 0; j < dimensionMatrice_; j++)
            nouvelle[i][j] = matrice_[i][j];
        matrice_ = nouvelle;
        for (int i = 0; i < dimensionMatrice_; i++)
          nouveau[i] = sommets_[i];
        sommets_ = nouveau;
      }
    }
/*-*/
/*+GrapheNonOrienteMatrice/ajouterSommet.java*/
  public Sommet ajouterSommet() {
    int numeroSommet;

    if (!sommetsLibres_.isEmpty()) {
      numeroSommet = ((Integer)sommetsLibres_.peek()).intValue();
      sommetsLibres_.pop();
    }
    else {
      ajusterMatrice(dimensionMatrice_ + 1);
      numeroSommet = dimensionMatrice_;
      dimensionMatrice_++;
    }
    for( int i = 0; i < dimensionMatrice_; i++) {
      ListeRecChainee<Arete> aretes = new ListeRecChainee<Arete>();
      matrice_[numeroSommet][i] = aretes;
      matrice_[i][numeroSommet] = aretes;
    }
    sommets_[numeroSommet] = new SommetGNOM(numeroSommet, this);
    nombreDeSommets_++;
    return  sommets_[numeroSommet];
  }
/*-*/
/*+GrapheNonOrienteMatrice/supprimerSommet.java*/
  public void supprimerSommet(Sommet s) {
    SommetGNOM sp = (SommetGNOM)s;

    prerequis(sp.proprietaire_ == this);
    for( int i = 0; i < dimensionMatrice_; i++) {
      ListeRecChainee<Arete> aretes = matrice_[sp.numero][i];
      for(Curseur<Arete> c = aretes.debut();
          c != null; c = aretes.suivant(c)  ) {
        AreteGNOM a =  (AreteGNOM)aretes.element(c);
        a.proprietaire_ = null;
      }
      matrice_[sp.numero][i] = null;
      matrice_[i][sp.numero] = null;
   }
    sommetsLibres_.push(sp.numero);
    sommets_[sp.numero] = null;
    sp.proprietaire_ = null;
    nombreDeSommets_--;
  }
/*-*/
/*+GrapheNonOrienteMatrice/ajouterArete.java*/
  public Arete ajouterArete(Sommet sommet1, Sommet sommet2, Number cout) {
    SommetGNOM sommet1p = (SommetGNOM)sommet1;
    SommetGNOM sommet2p = (SommetGNOM)sommet2;

    prerequis(sommet1p.proprietaire_ == this);
    prerequis(sommet2p.proprietaire_ == this);
    AreteGNOM arete = new AreteGNOM(sommet1p, sommet2p, cout, this);
    matrice_[sommet1p.numero][sommet2p.numero].ajouterDebut(arete);
    return arete;
  }
/*-*/
/*+GrapheNonOrienteMatrice/supprimerArete.java*/
  public void supprimerArete(Arete a) {
    AreteGNOM ap = (AreteGNOM)a;

    prerequis(ap.proprietaire_ == this);
    ap.proprietaire_ = null;
    ListeRecChainee<Arete> aretes = 
      matrice_[ap.sommet1_.numero][ap.sommet2_.numero];
    aretes.enlever(a);
  }
/*-*/
/*+GrapheNonOrienteMatrice/valuerArete.java*/
  public void valuerArete(Arete a, Number valeur) {
    AreteGNOM ap = (AreteGNOM)a;

    prerequis(ap.proprietaire_ == this);
    ap.cout_ = valeur;
  }
/*-*/
/*+GrapheNonOrienteMatrice/areteExiste.java*/
  public boolean areteExiste(Sommet s1, Sommet s2) {
    SommetGNOM s1p = (SommetGNOM)s1;
    SommetGNOM s2p = (SommetGNOM)s2;

    prerequis(s1p.proprietaire_ == this);
    prerequis(s2p.proprietaire_ == this);
    ListeRecChainee aretes = matrice_[s1p.numero][s2p.numero];
    return !aretes.estvide();
  }
/*-*/
/*+GrapheNonOrienteMatrice/degre.java*/
  public int degre(Sommet s) {
    SommetGNOM sp = (SommetGNOM)s;
    int degre = 0;

    prerequis(sp.proprietaire_ == this);
    for (int i = 0; i < dimensionMatrice_; i++) {
      if ( matrice_[i][sp.numero] != null)
        degre += matrice_[i][sp.numero].longueur();
    }
    return degre;
  }
/*-*/
/*+GrapheNonOrienteMatrice/nombreDeSommets.java*/
  public int nombreDeSommets() {
    return nombreDeSommets_;
  }
/*-*/
/*+GrapheNonOrienteMatrice/premiereArete.java*/
  public Curseur<Arete> premiereArete(Sommet s) {
    SommetGNOM sp = (SommetGNOM)s;

    prerequis(sp.proprietaire_ == this);
    for (int sommet2 = 0; sommet2 < dimensionMatrice_; sommet2++) {
      ListeRec<Arete> aretes = matrice_[sp.numero][sommet2];
      if (aretes != null && !aretes.estvide()) {
        return new CurseurSuccesseur(sp.numero, sommet2, aretes.debut());
      }
    }
    return null;
  }
/*-*/
/*+GrapheNonOrienteMatrice/areteSuivante.java*/
  public Curseur<Arete> areteSuivante(Curseur<Arete> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    Curseur<Arete> arete;

    ListeRec<Arete> aretes = matrice_[cs.sommet1][cs.sommet2];
    arete = aretes.suivant(cs.arete);
    if (arete != null)
       return new CurseurSuccesseur(cs.sommet1,cs.sommet2,arete);
    for (int sommet2 = cs.sommet2 + 1;
         sommet2 < dimensionMatrice_; sommet2++) {
      aretes = matrice_[cs.sommet1][sommet2];
      if (aretes != null && !aretes.estvide())
        return new CurseurSuccesseur(cs.sommet1, sommet2, aretes.debut());
    }
    return null;
  }
/*-*/
/*+GrapheNonOrienteMatrice/arete.java*/
  public Arete arete(Curseur<Arete> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    return (Arete)matrice_[cs.sommet1][cs.sommet2].element(cs.arete);
  }
/*-*/
/*+GrapheNonOrienteMatrice/premierSommet.java*/
  public Curseur<Sommet> premierSommet() {
    for (int i = 0; i < dimensionMatrice_; i++) {
      if (sommets_[i] != null) return new CurseurSommet(i);
    }
    return null;
  }
/*-*/
/*+GrapheNonOrienteMatrice/sommetSuivant.java*/
  public Curseur<Sommet> sommetSuivant(Curseur<Sommet> c) {
    CurseurSommet cs = (CurseurSommet)c;

    for (int i = cs.numero + 1; i < dimensionMatrice_; i++) {
      if (sommets_[i] != null) return new CurseurSommet(i);
    }
    return null;
  }
/*-*/
/*+GrapheNonOrienteMatrice/sommet.java*/
  public Sommet sommet(Curseur c) {
    CurseurSommet cs = (CurseurSommet)c;

    return sommets_[cs.numero];
  }
/*-*/
/*+GrapheNonOrienteMatrice/SommetGNOM.java*/
  private class SommetGNOM extends Sommet {
    public int numero;
    public GrapheNonOrienteMatrice proprietaire_;

    public SommetGNOM(int num, GrapheNonOrienteMatrice g) {
      numero = num;
      proprietaire_ = g;
    }
  }
/*-*/
/*+GrapheNonOrienteMatrice/AreteGNOM.java*/
  private class AreteGNOM extends Arete {
    public SommetGNOM sommet1_;
    public SommetGNOM sommet2_;
    public Number cout_;
    public GrapheNonOrienteMatrice proprietaire_;

    public AreteGNOM(SommetGNOM sommet1, SommetGNOM sommet2, Number cout,
                     GrapheNonOrienteMatrice g) {
      sommet1_ = sommet1;
      sommet2_ = sommet2;
      cout_ = cout;
      proprietaire_ = g;
    }

    public Sommet sommet1() {
      return sommet1_;
    }

    public Sommet sommet2() {
      return sommet2_;
    }

    public Number cout() {
      return cout_;
    }
  }
/*-*/
/*+GrapheNonOrienteMatrice/CurseurSuccesseur.java*/
  private class CurseurSuccesseur extends Curseur<Arete> {
    public int sommet1;
    public int sommet2;
    public Curseur<Arete> arete;

    public CurseurSuccesseur(int dep, int arr, Curseur<Arete> a) {
      sommet1 = dep;
      sommet2 = arr;
      arete = a;
    }

  }
/*-*/
/*+GrapheNonOrienteMatrice/CurseurSommet.java*/
  private class CurseurSommet extends Curseur<Sommet> {
    public int numero;

    public CurseurSommet(int num) {
      numero = num;
    }
  }


}


