package collections;
import java.util.HashMap;
import java.io.*;
import java.util.HashMap;
import java.util.StringTokenizer;


/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */
/*+GrapheOriente/graphe.java*/
public abstract class GrapheOriente {
  public abstract Sommet ajouterSommet();
  public abstract void supprimerSommet(Sommet s);
  public abstract Arc ajouterArc(Sommet s1, Sommet s2, Number valeur);
  public abstract void supprimerArc(Arc a);
  public abstract void valuerArc(Arc a, Number valeur);
  public abstract boolean arcExiste(Sommet s1, Sommet s2);
  public abstract int degreInterieur(Sommet s);
  public abstract int degreExterieur(Sommet s);
  public abstract int nombreDeSommets();
  public abstract Curseur<Arc> premierArc(Sommet s);
  public abstract Curseur<Arc> arcSuivant(Curseur<Arc> c);
  public abstract Arc arc(Curseur<Arc> c);
  public abstract Curseur<Sommet> premierSommet();
  public abstract Curseur<Sommet> sommetSuivant(Curseur<Sommet> c);
  public abstract Sommet sommet(Curseur<Sommet> c);
/*P+*/
  protected void prerequis(boolean b) {
    if (!b) {
      throw new Error("Prrequis non valide dans " + getClass().getName());
    }
  }

  public Object clone() {
    GrapheOriente copie;
    try {
      copie = (GrapheOriente) getClass().newInstance();
    } catch (Exception e) {
      throw new Error("Exception");
    }
    for(Curseur<Sommet> c = premierSommet(); c != null; c = sommetSuivant(c)) {
      Sommet s = sommet(c);
      Sommet sc = copie.ajouterSommet();
      sc.donneesUtilisateur = s.donneesUtilisateur;
      s.donneesUtilisateur = new DonneesClonage(sc, s.donneesUtilisateur);
    }
    for(Curseur<Sommet> c = premierSommet(); c != null; c = sommetSuivant(c)) {
      Sommet s = sommet(c);
      for (Curseur<Arc> ca = premierArc(s); ca != null; ca = arcSuivant(ca)) {
        Arc a = arc(ca);
        Sommet s1 = a.depart();
        Sommet s2 = a.arrivee();
        Arc ac = copie.ajouterArc(((DonneesClonage)s1.donneesUtilisateur).sommetClone,((DonneesClonage)s2.donneesUtilisateur).sommetClone, a.cout());
      }
    }
    for(Curseur<Sommet> c = premierSommet(); c != null; c = sommetSuivant(c)) {
      Sommet s = sommet(c);
      s.donneesUtilisateur = ((DonneesClonage)s.donneesUtilisateur).donneesUtilisateur;
    }
    return copie;
/*P-*/
  }
/*-*/
  public HashMap<String, Sommet> lire(String nomFichier)
         throws java.io.FileNotFoundException, java.io.IOException {
    FileReader flux = new FileReader(nomFichier);
    BufferedReader buf = new BufferedReader(flux);
    String line = null;
    HashMap<String, Sommet> sommets = new HashMap<String, Sommet>();
    Sommet sd, sa;

    while ((line = buf.readLine()) != null) {
      // on ignore les lignes vides
      if (line.compareTo("") == 0)  continue;
      // ainsi que celles qui dbutent par '#' (commentaires)
      if (line.charAt(0) == '#')  continue;
      StringTokenizer toks = new StringTokenizer(line, " [,]");
      String depart = toks.nextToken();
      String arrivee = toks.nextToken();
      int cout = Integer.decode(toks.nextToken()).intValue();
      if (sommets.containsKey(depart))
        sd = (Sommet)sommets.get(depart);
      else {
        sd = ajouterSommet();
        sd.donneesUtilisateur = depart;
        sommets.put(depart,sd);
      }
      if (sommets.containsKey(arrivee))
        sa = (Sommet)sommets.get(arrivee);
      else {
        sa = ajouterSommet();
        sa.donneesUtilisateur = arrivee;
        sommets.put(arrivee,sa);
      }
      Arc arc = ajouterArc(sd,sa, new Integer(cout));
      arc.donneesUtilisateur = "(" + sd.donneesUtilisateur + "," + sa.donneesUtilisateur + ")";
    }
    return sommets;
  }
  private class DonneesClonage {
    public Object donneesUtilisateur;
    public Sommet sommetClone;

    public DonneesClonage(Sommet s, Object donnees) {
      sommetClone = s;
      donneesUtilisateur = donnees;
    }
  }
}
