package collections;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+GrapheOrienteListeChainee/attributs.java*/
public class GrapheOrienteListeChainee extends GrapheOriente {
  private ListeRec<Sommet> sommets_;
/*-*/
/*+GrapheOrienteListeChainee/constructeur.java*/
  public GrapheOrienteListeChainee() {
    sommets_ = new ListeRecChainee<Sommet>();
  }
/*-*/
/*+GrapheOrienteListeChainee/ajouterSommet.java*/
  public Sommet ajouterSommet() {
    Sommet s = new SommetGOLC(this);
    sommets_.ajouterDebut(s);
    return s;
  }
/*-*/
/*+GrapheOrienteListeChainee/supprimerSommet.java*/
  public void supprimerSommet(Sommet s) {
    SommetGOLC sp = (SommetGOLC)s;

    prerequis(sp.proprietaire_ == this);
    sommets_.enlever(s);
    for (Curseur<Sommet> cs = sommets_.debut();
         cs != null; cs = sommets_.suivant(cs)) {
      ListeRec<Arc> arcs = ((SommetGOLC)sommets_.element(cs)).successeurs;
      Curseur<Arc> cour, prec;
      for (prec = null, cour = arcs.debut(); cour != null; ) {
        Arc a = (Arc)arcs.element(cour);
        cour = arcs.suivant(cour);
        if (a.arrivee() == s) {
          if (prec == null) {
            arcs.enleverPremier();
          }
          else {
            arcs.supprimerSuivant(prec);
          }
        }
        else
          prec = cour;
      }
    }
  }
/*-*/
/*+GrapheOrienteListeChainee/ajouterArc.java*/
  public Arc ajouterArc(Sommet depart, Sommet arrivee, Number cout) {
    SommetGOLC departp = (SommetGOLC)depart;
    SommetGOLC arriveep = (SommetGOLC)arrivee;

    prerequis(departp.proprietaire_ == this);
    prerequis(arriveep.proprietaire_ == this);
    ArcGOLC arc = new ArcGOLC(departp, arriveep, cout, this);
    departp.successeurs.ajouterDebut(arc);
    arriveep.degreInterieur++;
    return arc;
  }
/*-*/
/*+GrapheOrienteListeChainee/supprimerArc.java*/
  public void supprimerArc(Arc a) {
    ArcGOLC ap = (ArcGOLC)a;
    SommetGOLC depart = (SommetGOLC)a.depart();
    SommetGOLC arrivee = (SommetGOLC)a.arrivee();

    prerequis(ap.proprietaire_ == this);
    ListeRec<Arc> arcs = depart.successeurs;
    arcs.enlever(a);
    arrivee.degreInterieur--;
  }
/*-*/
/*+GrapheOrienteListeChainee/valuerArc.java*/
  public void valuerArc(Arc a, Number valeur) {
    ArcGOLC ap = (ArcGOLC)a;

    prerequis(ap.proprietaire_ == this);
    ap.cout_ = valeur;
  }
/*-*/
/*+GrapheOrienteListeChainee/arcExiste.java*/
  public boolean arcExiste(Sommet depart, Sommet arrivee) {
    SommetGOLC departp = (SommetGOLC)depart;
    SommetGOLC arriveep = (SommetGOLC)arrivee;

    prerequis(departp.proprietaire_ == this);
    prerequis(arriveep.proprietaire_ == this);
    ListeRec<Arc> arcs = departp.successeurs;
    for (Curseur<Arc> c = arcs.debut(); c != null; c = arcs.suivant(c)) {
      Arc a = arcs.element(c);
      if (a.arrivee() == arrivee) return true;
    }
    return false;
  }
/*-*/
/*+GrapheOrienteListeChainee/degreInterieur.java*/
  public int degreInterieur(Sommet s) {
    SommetGOLC sp = (SommetGOLC)s;

    prerequis(sp.proprietaire_ == this);
    return sp.degreInterieur;
  }
/*-*/
/*+GrapheOrienteListeChainee/degreExterieur.java*/
  public int degreExterieur(Sommet s) {
    SommetGOLC sp = (SommetGOLC)s;

    prerequis(sp.proprietaire_ == this);
    return sp.successeurs.longueur();
  }
/*-*/
/*+GrapheOrienteListeChainee/nombreDeSommets.java*/
  public int nombreDeSommets() {
    return sommets_.longueur();
  }
/*-*/
/*+GrapheOrienteListeChainee/premierArc.java*/
  public Curseur<Arc> premierArc(Sommet s) {
    SommetGOLC sp = (SommetGOLC)s;
    prerequis(sp.proprietaire_ == this);
    if (sp.successeurs.debut() == null) return null;
    return new CurseurSuccesseur(sp, sp.successeurs.debut());
  }
/*-*/
/*+GrapheOrienteListeChainee/arcSuivant.java*/
  public Curseur<Arc> arcSuivant(Curseur<Arc> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    Curseur<Arc> suivant = cs.depart.successeurs.suivant(cs.arc);
    if (suivant == null) return null;
    return new CurseurSuccesseur(cs.depart,suivant);
  }
/*-*/
/*+GrapheOrienteListeChainee/arc.java*/
  public Arc arc(Curseur<Arc> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    return (Arc)cs.depart.successeurs.element(cs.arc);
  }
/*-*/
/*+GrapheOrienteListeChainee/premierSommet.java*/
  public Curseur<Sommet> premierSommet() {
    return sommets_.debut();
  }
/*-*/
/*+GrapheOrienteListeChainee/sommetSuivant.java*/
  public Curseur<Sommet> sommetSuivant(Curseur<Sommet> c) {
    return sommets_.suivant(c);
  }
/*-*/
/*+GrapheOrienteListeChainee/sommet.java*/
  public Sommet sommet(Curseur<Sommet> c) {
    return (Sommet)sommets_.element(c);
  }
/*-*/
/*+GrapheOrienteListeChainee/SommetGOLC.java*/
  private class SommetGOLC  extends Sommet {
    public ListeRecChainee<Arc> successeurs;
    public int degreInterieur;
    public GrapheOrienteListeChainee proprietaire_;

    public SommetGOLC(GrapheOrienteListeChainee g) {
      successeurs = new ListeRecChainee<Arc>();
      proprietaire_ = g;
      degreInterieur = 0;
    }
  }
/*-*/
/*+GrapheOrienteListeChainee/ArcGOLC.java*/
  private class ArcGOLC extends Arc {
    public SommetGOLC depart_;
    public SommetGOLC arrivee_;
    public GrapheOrienteListeChainee proprietaire_;
    public Number cout_;

    public ArcGOLC(SommetGOLC depart, SommetGOLC arrivee, 
                   Number cout, GrapheOrienteListeChainee g) {
      depart_ = depart;
      arrivee_ = arrivee;
      cout_ = cout;
      proprietaire_ = g;
    }
    public Sommet depart() {
      return depart_;
    }
    public Sommet arrivee() {
      return arrivee_;
    }
    public Number cout() {
      return cout_;
    }
  }
/*-*/
/*+GrapheOrienteListeChainee/CurseurSuccesseur.java*/
  private class CurseurSuccesseur extends Curseur<Arc> {
    public SommetGOLC depart;
    public Curseur<Arc> arc;

    public CurseurSuccesseur(SommetGOLC dep, Curseur<Arc> a) {
      depart = dep;
      arc = a;
    }
  }
/*-*/
}