package collections;
import java.util.Vector;
import java.util.Stack;

/**
 * <p>Title: Collections</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2003</p>
 * <p>Company: Suplec</p>
 * @author Claude Bocage
 * @version 1.0
 */

/*+GrapheOrienteMatrice/attributs.java*/
public class GrapheOrienteMatrice extends GrapheOriente {
  private Sommet sommets_[];
  private ListeRecChainee<Arc>[][] matrice_;
  private Stack<Integer> sommetsLibres_;
  private int dimensionMatrice_;
  private int nombreDeSommets_;
  private final int DELTA = 10;
/*-*/
  @SuppressWarnings("unchecked")
  private ListeRecChainee<Arc>[][] creerMatrice(int taille) {
    int[] dimensions = {taille, taille};
    return (ListeRecChainee<Arc>[][])
      java.lang.reflect.Array.newInstance(ListeRecChainee.class, dimensions);
  }

/*+GrapheOrienteMatrice/constructeur.java*/
  public GrapheOrienteMatrice() {
    matrice_ = creerMatrice(DELTA);
    sommets_ = new Sommet[DELTA];
    dimensionMatrice_ = 0;
    nombreDeSommets_ = 0;
    sommetsLibres_ = new Stack<Integer>();
  }
/*-*/

/*+GrapheOrienteMatrice/ajusterMatrice.java*/
  private void ajusterMatrice(int dimension) {
    if (matrice_.length < dimension) {
      int nd = matrice_.length + DELTA;
      ListeRecChainee<Arc>[][] nouvelle = creerMatrice(nd);
      Sommet[] nouveau = new Sommet[nd];
      for (int i = 0; i < dimensionMatrice_; i++)
        for (int j = 0; j < dimensionMatrice_; j++)
          nouvelle[i][j] = matrice_[i][j];
      matrice_ = nouvelle;
      for (int i = 0; i < dimensionMatrice_; i++)
        nouveau[i] = sommets_[i];
      sommets_ = nouveau;
    }
  }
/*-*/

/*+GrapheOrienteMatrice/ajouterSommet.java*/
  public Sommet ajouterSommet() {
    int numeroSommet;

    if (!sommetsLibres_.isEmpty()) {
      numeroSommet = ((Integer)sommetsLibres_.peek()).intValue();
      sommetsLibres_.pop();
    }
    else {
      ajusterMatrice(dimensionMatrice_ + 1);
      numeroSommet = dimensionMatrice_;
      dimensionMatrice_++;
    }
    for (int i = 0; i < dimensionMatrice_; i++) {
      matrice_[numeroSommet][i] = new ListeRecChainee<Arc>();
      matrice_[i][numeroSommet] = new ListeRecChainee<Arc>();
    }
    sommets_[numeroSommet] = new SommetGOM(numeroSommet, this);
    nombreDeSommets_++;
    return sommets_[numeroSommet];
  }
/*-*/

/*+GrapheOrienteMatrice/supprimerSommet.java*/
  public void supprimerSommet(Sommet s) {
    SommetGOM sp = (SommetGOM)s;

    prerequis(sp.proprietaire_ == this);
    for( int i = 0; i < dimensionMatrice_; i++) {
      ListeRecChainee<Arc> arcs = matrice_[sp.numero][i];
      for(Curseur<Arc> c = arcs.debut(); c != null; c = arcs.suivant(c)  ) {
        ArcGOM a =  (ArcGOM)arcs.element(c);
        a.proprietaire_ = null;
      }
      matrice_[sp.numero][i] = null;
    }
    for( int i = 0; i < dimensionMatrice_; i++) {
      ListeRecChainee<Arc> arcs = matrice_[i][sp.numero];
      if (arcs == null) continue;
      for(Curseur<Arc> c = arcs.debut(); c != null; c = arcs.suivant(c)  ) {
        ArcGOM a =  (ArcGOM)arcs.element(c);
        a.proprietaire_ = null;
      }
      matrice_[i][sp.numero] = null;
    }
    sommetsLibres_.push(sp.numero);
    sommets_[sp.numero] = null;
    sp.proprietaire_ = null;
    nombreDeSommets_--;
  }
/*-*/

/*+GrapheOrienteMatrice/ajouterArc.java*/
  public Arc ajouterArc(Sommet depart, Sommet arrivee, Number cout) {
    SommetGOM departp = (SommetGOM)depart;
    SommetGOM arriveep = (SommetGOM)arrivee;

    prerequis(departp.proprietaire_ == this);
    prerequis(arriveep.proprietaire_ == this);
    ArcGOM arc = new ArcGOM(departp, arriveep, cout, this);
    matrice_[departp.numero][arriveep.numero].ajouterDebut(arc);
    return arc;
  }
/*-*/

/*+GrapheOrienteMatrice/supprimerArc.java*/
  public void supprimerArc(Arc a) {
    ArcGOM ap = (ArcGOM)a;

    prerequis(ap.proprietaire_ == this);
    ap.proprietaire_ = null;
    ListeRecChainee<Arc> arcs =
        matrice_[ap.depart_.numero][ap.arrivee_.numero];
    arcs.enlever(a);
  }
/*-*/

/*+GrapheOrienteMatrice/valuerArc.java*/
  public void valuerArc(Arc a, Number valeur) {
    ArcGOM ap = (ArcGOM)a;

    prerequis(ap.proprietaire_ == this);
    ap.cout_ = valeur;
  }
/*-*/

/*+GrapheOrienteMatrice/arcExiste.java*/
  public boolean arcExiste(Sommet s1, Sommet s2) {
    SommetGOM s1p = (SommetGOM)s1;
    SommetGOM s2p = (SommetGOM)s2;

    prerequis(s1p.proprietaire_ == this);
    prerequis(s2p.proprietaire_ == this);
    ListeRecChainee<Arc> arcs = matrice_[s1p.numero][s2p.numero];
    return !arcs.estvide();
  }
/*-*/

/*+GrapheOrienteMatrice/degreInterieur.java*/
  public int degreInterieur(Sommet s) {
    SommetGOM sp = (SommetGOM)s;
    int degre = 0;

    prerequis(sp.proprietaire_ == this);
    for (int i = 0; i < dimensionMatrice_; i++) {
      if ( matrice_[i][sp.numero] != null)
        degre += matrice_[i][sp.numero].longueur();
    }
    return degre;
  }
/*-*/

/*+GrapheOrienteMatrice/degreExterieur.java*/
  public int degreExterieur(Sommet s) {
    SommetGOM sp = (SommetGOM)s;
    int degre = 0;

    prerequis(sp.proprietaire_ == this);
    for (int i = 0; i < dimensionMatrice_; i++) {
      if ( matrice_[sp.numero][i] != null)
        degre += matrice_[sp.numero][i].longueur();
    }
    return degre;
  }
/*-*/

/*+GrapheOrienteMatrice/nombreDeSommets.java*/
  public int nombreDeSommets() {
    return nombreDeSommets_;
  }
/*-*/
/*+GrapheOrienteMatrice/premierArc.java*/
  public Curseur<Arc> premierArc(Sommet s) {
    SommetGOM sp = (SommetGOM)s;

    prerequis(sp.proprietaire_ == this);
    for (int arrivee = 0; arrivee < dimensionMatrice_; arrivee++) {
      ListeRec<Arc> arcs = matrice_[sp.numero][arrivee];
      if (arcs != null && !arcs.estvide()) {
        return new CurseurSuccesseur(sp.numero, arrivee, arcs.debut());
      }
    }
    return null;
  }
/*-*/
/*+GrapheOrienteMatrice/arcSuivant.java*/
  public Curseur<Arc> arcSuivant(Curseur<Arc> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    Curseur<Arc> arc;

    ListeRec<Arc> arcs = matrice_[cs.depart][cs.arrivee];
    arc = arcs.suivant(cs.arc);
    if (arc != null)
       return new CurseurSuccesseur(cs.depart,cs.arrivee,arc);
    for (int arrivee = cs.arrivee + 1;
         arrivee < dimensionMatrice_; arrivee++) {
      arcs = matrice_[cs.depart][arrivee];
      if (arcs != null && !arcs.estvide())
        return new CurseurSuccesseur(cs.depart, arrivee, arcs.debut());
    }
    return null;
  }
/*-*/
/*+GrapheOrienteMatrice/arcc.java*/
  public Arc arc(Curseur<Arc> c) {
    CurseurSuccesseur cs = (CurseurSuccesseur)c;
    return (Arc)matrice_[cs.depart][cs.arrivee].element(cs.arc);
  }
/*-*/
/*+GrapheOrienteMatrice/premierSommet.java*/
  public Curseur<Sommet> premierSommet() {
    for (int i = 0; i < dimensionMatrice_; i++) {
      if (sommets_[i] != null) return new CurseurSommet(i);
    }
    return null;
  }
/*-*/
/*+GrapheOrienteMatrice/sommetSuivant.java*/
  public Curseur<Sommet> sommetSuivant(Curseur c) {
    CurseurSommet cs = (CurseurSommet)c;

    for (int i = cs.numero + 1; i < dimensionMatrice_; i++) {
      if (sommets_[i] != null) return new CurseurSommet(i);
    }
    return null;
  }
/*-*/
/*+GrapheOrienteMatrice/sommetc.java*/
  public Sommet sommet(Curseur<Sommet> c) {
    CurseurSommet cs = (CurseurSommet)c;

    return sommets_[cs.numero];
  }
/*-*/

/*+GrapheOrienteMatrice/sommet.java*/
  private class SommetGOM extends Sommet {
    public int numero;
    public GrapheOrienteMatrice proprietaire_;

    public SommetGOM(int num, GrapheOrienteMatrice g) {
      numero = num;
      proprietaire_ = g;
    }
  }
/*-*/

/*+GrapheOrienteMatrice/arc.java*/
  private class ArcGOM extends Arc {
    public SommetGOM depart_;
    public SommetGOM arrivee_;
    public Number cout_;
    public GrapheOrienteMatrice proprietaire_;

    public ArcGOM(SommetGOM depart, SommetGOM arrivee, Number cout,
                  GrapheOrienteMatrice g) {
      depart_ = depart;
      arrivee_ = arrivee;
      cout_ = cout;
      proprietaire_ = g;
    }
    public Sommet depart() {
      return depart_;
    }
    public Sommet arrivee() {
      return arrivee_;
    }
    public Number cout() {
      return cout_;
    }
  }
/*-*/
/*+GrapheOrienteMatrice/CurseurSuccesseur.java*/
  private class CurseurSuccesseur extends Curseur<Arc> {
    public int depart;
    public int arrivee;
    public Curseur<Arc> arc;

    public CurseurSuccesseur(int dep, int arr, Curseur<Arc> a) {
      depart = dep;
      arrivee = arr;
      arc = a;
    }

  }
/*-*/
/*+GrapheOrienteMatrice/CurseurSommet.java*/
  private class CurseurSommet extends Curseur<Sommet> {
    public int numero;

    public CurseurSommet(int num) {
      numero = num;
    }
  }


}


