/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "ListeIter.java"
 *                                    created: 2003-02-10 10:51:19
 *                                last update: 2007-02-16 17:01:12
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *  Liste itrative, implmentation avec oprations qui modifient la liste.
 *
 * <pre>
 * spc LISTE_ITER
 *   tend ELEMENT, POSITION, TAILLE
 *   sorte Liste
 *   oprations
 *     listevide :                        -> Liste
 *     longueur  : Liste                  -> Taille
 *     insrer   : Liste Position lment -> Liste
 *     supprimer : Liste Position         -> Liste
 *     ime      : Liste Position         -> lment
 *   prconditions l:Liste; p:Position; e:lment
 *     pr insrer(l,p,e) = (p >= 1) .et. (p <= longueur(l) + 1)
 *     pr supprimer(l,p) = (p >= 1) .et. (p <= longueur(l))
 *     pr ime(l,p)      = (p >= 1) .et. (p <= longueur(l))
 *   axiomes l:Liste; p1,p2,p3:Position; e,e1,e2:lment
 *     insrer(insrer(l,p1,e1),p2,e2) = si p1 < p2 alors
 *                                         insrer(insrer(l,p2-1,e2),p1,e1)
 *                                       sinon
 *                                         insrer(insrer(l,p2,e2),p1+1,e1)
 *                                       fin si
 *     supprimer(insrer(l,p,e),p) = l
 *     longueur(listevide) = 0
 *     longueur(insrer(l,p,e)) = longueur(l+1)
 *     ime(insrer(l,p,e),p) = e
 *   fspc
 * </pre>
 *
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2003-02-10 FBO 1.0 original
 * ###################################################################
 */
package collections;

/**
 * Reprsente une liste supportant les oprations de
 * la spcification itrative LISTE_ITER.
 */
/*+ListeIter/classe.java*/
public abstract class ListeIter<T> extends Liste<T> {
  // Oprations de la spcification.
  // Le constructeur tiendra lieu de constante "listevide"
  /**
   * Insre l'lment <code>e</code>  la position <code>p</code>
   * dans la liste. <code>p</code> doit tre compris entre 1
   * (insertion en dbut de liste) et <code>longueur()</code>+1
   * (insertion en fin de liste).
   */
  public abstract void inserer(int p, T e);
  /** Supprime le <code>p</code><sup></sup> lment de la liste. */
  public abstract void supprimer(int p);
  /** Rend le <code>p</code><sup></sup> lment de la liste. */
  public abstract T ieme(int p);
  /** Rend le nombre d'lments prsents dans la liste. */
  public abstract int longueur();
/*P+*/
  public void afficher() {
    for (int i = 1; i <= longueur();i++)
      System.out.print(ieme(i) + " ");
    System.out.println();
  }

  public void vider() {
    while(longueur() != 0)
      supprimer(1);
  }

  public ListeIter<T> dupliquer() throws java.lang.IllegalAccessException,java.lang.InstantiationException {
/*     ListeIter<T> duplicata = (ListeIter<T>)getClass().newInstance(); */
    ListeIter<T> duplicata = makeInstance();
    for (int i = 1; i <= longueur(); i++)
      duplicata.inserer(i, ieme(i));
    return duplicata;
  }

  public boolean equals(ListeIter<T> autre) {
    if (longueur() != autre.longueur()) return false;
    for (int i = 1; i <= longueur(); i++) {
      if (ieme(i) != autre.ieme(i)) return false;
    }
    return true;
  }
  protected abstract ListeIter<T> makeInstance();
/*P-*/
}
/*-*/
