/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "ListeIterChainee.java"
 *                                    created: 2003-02-10 12:29:49
 *                                last update: 2008-02-27 10:03:01
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *  Implmentation par liste chane de la liste itrative.
 *
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2003-02-10 FBO 1.0 original
 * ###################################################################
 */
package collections;

/**
 * Liste itrative dont les lments sont rangs dans une liste chane.
 */
/*+ListeIterChainee/classe.java*/
public class ListeIterChainee<T> extends ListeIter<T> {
  /** Maillon portant le premier lment de la liste. */
  private Maillon premier_;

  /** Nombre d'lments dans la liste. */
  private int longueur_;
/*-*/
  // Oprations de la spcification.

  /** Constructeur = listevide. */
/*+ListeIterChainee/listevide.java*/
  public ListeIterChainee() {
    premier_ = null;
    longueur_ = 0;
  }
/*-*/

  /** Rend le nombre d'lments contenus dans la liste. */
/*+ListeIterChainee/longueur.java*/
  public int longueur() {
    return longueur_;
  }
/*-*/

/*+ListeIterChainee/inserer.java*/
  public void inserer(int p, T e) {
    prerequis((p >= 1) && (p <= longueur_ + 1));
    Maillon courant = premier_;
    Maillon precedent = null;
    for (int i = 1; i < p; i++) {
      precedent = courant;
      courant = courant.suivant;
    }
    Maillon nouveau = new Maillon(e, courant);
    if (precedent == null) {
      premier_ = nouveau;
    } else {
      precedent.suivant = nouveau;
    }
    longueur_++;
  }
/*-*/

/*+ListeIterChainee/supprimer.java*/
  public void supprimer(int p) {
    prerequis((p >= 1) && (p <= longueur_));
    Maillon courant = premier_;
    Maillon precedent = null;
    for (int i = 1; i < p; i++) {
      precedent = courant;
      courant = courant.suivant;
    }
    if (precedent == null) {
      premier_ = courant.suivant;
    } else {
      precedent.suivant = courant.suivant;
    }
    longueur_--;
  }
/*-*/

/*+ListeIterChainee/ieme.java*/
  public T ieme(int p) {
    prerequis((p >= 1) && (p <= longueur_));
    Maillon courant = premier_;
    for (int i = 1; i < p; i++) {
      courant = courant.suivant;
    }
    return courant.element;
  }
/*-*/

  /**
   * Implmente la notion de curseur : dans une liste reprsente par
   * liste chane, un curseur est simplement un maillon de la liste.
   */
/*+ListeIterChainee/classeMaillon.java*/
  private class Maillon extends Curseur<T> {
    /**
     * On ne protge pas les membres de cette classe. En effet,
     * <code>Maillon</code> est une classe prive qui ne peut tre
     * vue par aucune autre classe que <code>ListeIterChainee</code>.
     * De plus, son comportement est intimement li  celui de la liste.
     * Il est donc inutile de diminuer l'efficacit du code par des
     * appels de mthodes qui n'apportent rien au niveau de l'encapsulation
     * ou du polymorphisme.
     * Notez qu'il s'agit l d'une situation exceptionnelle.
     */
    public T element;
    public Maillon suivant;

    /**
     * Construit un maillon contenant <code>e</code> dont le maillon
     * suivant est <code>s</code>.
     */
    public Maillon(T e, Maillon s) {
      element = e;
      suivant = s;
    }

  }
/*-*/
  // Oprations sur les curseurs.
  //
  // Les oprations qui reoivent un Curseur devraient vrifier que
  // ce curseur appartient bien  la liste. Ici, cette vrification
  // n'est pas faite, et utiliser un curseur invalide provoquera donc
  // un comportement erratique sans qu'une erreur soit forcment signale.
  // Une solution consisterait  placer une rfrence sur la liste dans
  // laquelle le Curseur a un sens et  vrifier que cette liste est
  // la bonne  chaque appel. Cela viterait les comportements erratiques
  // silencieux mais dgraderait les performances.

/*+ListeIterChainee/methodesAvecCurseurs1.java*/
  public Curseur<T> debut() {
    return premier_;
  }

  public Curseur<T> suivant(Curseur<T> c) {
    Maillon m = (Maillon)c;
    return m.suivant;
  }

  public T element(Curseur<T> c) {
    Maillon m = (Maillon)c;
    return m.element;
  }

  public void remplacer(Curseur<T> c, T e) {
    Maillon m = (Maillon)c;
    m.element = e;
  }

  public void supprimerSuivant(Curseur<T> c) {
    Maillon m = (Maillon)c;
    if (m.suivant == null) {
      throw new Error("Curseur invalide pour supprimerSuivant()");
    }
    m.suivant = m.suivant.suivant;
    longueur_--;
  }
/*-*/
/*+ListeIterChainee/methodesAvecCurseurs2.java*/
  public void insererApres(Curseur<T> c, T e) {
    Maillon m = (Maillon)c;
    if ( m == null )
      premier_ = new Maillon(e, premier_);
    else
      m.suivant = new Maillon(e, m.suivant);
    longueur_++;
  }
/*-*/
  protected ListeIterChainee<T> makeInstance() {
    return new ListeIterChainee<T>();
  }
  // Mthode de test
  public static void main(String[] args) {
    ListeIter<Object> l = new ListeIterChainee<Object>();
    l.inserer(1,"a");
    ListeIter<String> l2 = new ListeIterChainee<String>();
    l2.inserer(1,"b");
    l2.inserer(2,"c");
    l.inserer(2,l2);
    l.inserer(3,"d");
    System.out.println(l.toString());
    l2.inserer(2,"b'");
    l.inserer(2,"a'");
    System.out.println(l.toString());
    l2.supprimer(1);
    l.supprimer(1);
    System.out.println(l.toString());
    Curseur<Object> c = l.debut();
    l.remplacer(c,"a");
    Curseur<String> c2 = l2.debut();
    l2.remplacer(c2,"b");
    System.out.println(l.toString());
    l.insererApres(c,"A");
    System.out.println(l.toString());
    // Provoque une erreur car c n'est pas un curseur du bon type pour l2!
    // l2.insererApres(c,"Z");
    System.out.println(l.toString());
  }
}
