/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "ListeIterTableau.java"
 *                                    created: 2003-02-10 11:14:41
 *                                last update: 2008-02-27 09:58:36
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *  Implmentation par tableau de la liste itrative.
 *
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2003-02-10 FBO 1.0 original
 * ###################################################################
 */
package collections;

import java.lang.reflect.Array;

/**
 * Liste itrative dont les lments sont rangs dans un tableau.
 */
/*+ListeIterTableau/attributs.java*/
public class ListeIterTableau<T> extends ListeIter<T> {
  /** Tableau des lments. */
  private Conteneur[] elems_;

  /** Nombre d'lments dans la liste. */
  private int longueur_;
/*-*/
  
  // Conteneur est un type paramtr, son type lors de l'excution
  // n'est pas le mme que lors de la compilation en Java, ce qui empche
  // de crer de tableaux de Conteneur. On cre  la place un tableau du
  // type de Conteneur  l'excution (Conteneur.class), que l'on convertit
  // en tableau de Conteneur. Le @SuppressWarning vite l'avertissement
  // d au forage de type lors de la compilation.
/*+ListeIterTableau/creerTableau.java*/
  @SuppressWarnings("unchecked")
  private Conteneur[] creerTableau(int taille) {
    return (Conteneur[])Array.newInstance(Conteneur.class, taille);
  }
/*-*/
  
  // Oprations de la spcification.
  // Constructeur = listevide
/*+ListeIterTableau/constructeur.java*/
  public ListeIterTableau() {
    elems_ = creerTableau(0);
    longueur_ = 0;
  }
/*-*/

/*+ListeIterTableau/longueur.java*/
  public int longueur() {
    return longueur_;
  }
/*-*/

/*+ListeIterTableau/inserer.java*/
  public void inserer(int p, T e) {
    prerequis((p >= 1) && (p <= longueur_ + 1));
    if (longueur_+1 > elems_.length) {
      Conteneur[] nouveau = creerTableau(elems_.length + 1);
      for (int i = 0; i < p-1; i++) {
        nouveau[i] = elems_[i];
      }
      nouveau[p-1] = new Conteneur(p-1,e);
      for(int i = p; i < longueur_+1; i++) {
        nouveau[i] = elems_[i-1];
        nouveau[i].idx = i;
      }
      elems_ = nouveau;
    } else {
      for(int i = longueur_; i > p-1; i--) {
        elems_[i] = elems_[i-1];
        elems_[i].idx = i;
      }
      elems_[p-1] = new Conteneur(p-1,e);
    }
    longueur_++;
  }
/*-*/

/*+ListeIterTableau/supprimer.java*/
  public void supprimer(int p) {
    prerequis((p >= 1) && (p <= longueur_));
    for (int i = p-1; i < longueur_-1; i++) {
      elems_[i] = elems_[i+1];
      elems_[i].idx = i;
    }
    elems_[longueur_ - 1] = null;
    longueur_--;
  }
/*-*/

/*+ListeIterTableau/ieme.java*/
  public T ieme(int p) {
    prerequis((p >= 1) && (p <= longueur_));
    return elems_[p-1].element;
  }
/*-*/

  /**
   * Implmente la notion de curseur : dans une liste reprsente par
   * tableau, un curseur est simplement un indice dans le tableau.
   */
/*+ListeIterTableau/conteneur.java*/
  private class Conteneur extends Curseur<T> {
    /**
     * On ne protge pas l'unique membre de cette classe. En effet,
     * <code>CurseurIter</code> est une classe prive qui ne peut tre
     * vue par aucune autre classe que <code>ListeIterTableau</code>.
     * De plus, son comportement est intimement li  celui de la liste.
     * Il est donc inutile de diminuer l'efficacit du code par des
     * appels de mthodes qui n'apportent rien au niveau de l'encapsulation
     * ou du polymorphisme.
     * Notez qu'il s'agit l d'une situation exceptionnelle.
     */
    public int idx;
    public T element;

    /** Construit un Curseur pointant sur l'lment <code>p</code>+1 */
    public Conteneur(int p, T e) {
      idx = p;
      element = e;
    }
  }
/*-*/
  // Oprations sur les curseurs.
  //
  // Les oprations qui reoivent un Curseur devraient vrifier que
  // ce curseur appartient bien  la liste. Ici, cette vrification
  // n'est pas faite, et utiliser un curseur invalide provoquera donc
  // un comportement erratique sans qu'une erreur soit forcment signale.
  // Une solution consisterait  placer une rfrence sur la liste dans
  // laquelle le Curseur a un sens et  vrifier que cette liste est
  // la bonne  chaque appel. Cela viterait les comportements erratiques
  // silencieux mais dgraderait les performances.

/*+ListeIterTableau/methodesAvecCurseurs1.java*/
  public Curseur<T> debut() {
    if (longueur_ == 0) {
      return null;
    }
    return elems_[0];
  }
/*-*/
/*+ListeIterTableau/methodesAvecCurseurs2.java*/
  public Curseur<T> suivant(Curseur<T> c) {
    Conteneur ci = (Conteneur)c;

    if (ci.idx == longueur_-1) {
      return null;
    } else {
      return elems_[ci.idx + 1];
    }
  }

  public T element(Curseur<T> c) {
    Conteneur ci = (Conteneur)c;
    return elems_[ci.idx].element;
  }

  public void remplacer(Curseur<T> c, T e) {
    Conteneur ci = (Conteneur)c;
    elems_[ci.idx].element = e;
  }

  public void supprimerSuivant(Curseur<T> c) {
    Conteneur ci = (Conteneur)c;
    supprimer(ci.idx + 2);
  }

  public void insererApres(Curseur<T> c, T e) {
    Conteneur ci = (Conteneur)c;
    if (ci == null)
      inserer(1, e);
    else
      inserer(ci.idx + 2, e);
  }
/*-*/
  protected ListeIterTableau<T> makeInstance() {
    return new ListeIterTableau<T>();
  }
  
  // Mthode de test
  public static void main(String[] args) {
    ListeIter<Object> l = new ListeIterTableau<Object>();
    l.inserer(1,"a");
    ListeIter<String> l2 = new ListeIterTableau<String>();
    l2.inserer(1,"b");
    l2.inserer(2,"c");
    l.inserer(2,l2);
    l.inserer(3,"d");
    System.out.println(l.toString());
    l2.inserer(2,"b'");
    l.inserer(2,"a'");
    System.out.println(l.toString());
    l2.supprimer(1);
    l.supprimer(1);
    System.out.println(l.toString());
    Curseur<Object> c = l.debut();
    l.remplacer(c,"a");
    Curseur<String> c2 = l2.debut();
    l2.remplacer(c2,"b");
    System.out.println(l.toString());
    l.insererApres(c,"A");
    System.out.println(l.toString());
    // Provoque une erreur car c n'est pas un curseur du bon type pour l2!
    // l2.insererApres(c,"Z");
    System.out.println(l.toString());
  }
}
