/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "ListeRec.java"
 *                                    created: 2003-02-10 10:51:32
 *                                last update: 2008-02-22 17:02:50
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *  Liste rcursive, implmentation avec oprations qui modifient la liste.
 *
 *  <pre>
 *  spc LISTE_REC
 *    tend ELEMENT, BOOL
 *    sorte Liste
 *    oprations
 *      listevide      :                -> Liste
 *      ajouterDbut   : Liste, lment -> Liste
 *      enleverPremier : Liste          -> Liste
 *      premier        : Liste          -> lment
 *      estvide        : Liste          -> Bool
 *    prconditions l:Liste
 *      pr premier(l) = .not. estvide(l)
 *      pr enleverPremier(l) = .not. estvide(l)
 *    axiomes l:Liste; e:lment
 *      estvide(listevide) = vrai
 *      estvide(ajouterDbut(l,e) = faux
 *      premier(ajouterDbut(l,e)) = e
 *      enleverPremier(ajouterDbut(l,e)) = l
 *    fspc
 *    </pre>
 *
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2003-02-10 FBO 1.0 original
 * ###################################################################
 */
package collections;

/**
 * Reprsente une liste supportant les oprations de la
 * spcification rcursive LISTE_REC.
 */
/*+ListeRec/classe.java*/
public abstract class ListeRec<T> extends Liste<T> {
  // Oprations de la spcification.
  // Le constructeur tiendra lieu de constante "listevide"

  /** Ajoute <code>e</code> en tte de la liste. */
  public abstract void ajouterDebut(T e);

  /** Supprime le premier lment de la liste. */
  public abstract void enleverPremier();

  /** Rend le premier lment de la liste. */
  public abstract T premier();

  /** Indique si la liste est vide. */
  public abstract boolean estvide();
  public abstract int longueur();
/*P+*/
  public void enlever(T e) {
    Curseur<T> prec;
    Curseur<T> cour;
    for (cour = debut(), prec = null; cour != null; prec = cour, cour = suivant(cour)) {
      if (element(cour) == e)
        break;
    }
    if (cour == null) return;
    if (prec != null)
      supprimerSuivant(prec);
    else
      enleverPremier();
  }

  public void afficher() {
    Curseur<T> c = debut();
    while (c != null) {
      System.out.print(element(c) + " ");
      c = suivant(c);
    }
    System.out.println();
  }

  public void vider() {
    while (!estvide()) {
      enleverPremier();
    }
  }

  public ListeRec<T> dupliquer() throws java.lang.IllegalAccessException,
                                        java.lang.InstantiationException {
    ListeRec<T> duplicata = makeInstance();
    dupliquerLoc(this, duplicata, debut());
    return duplicata;
  }

  private static <V> void dupliquerLoc(ListeRec<V> source, ListeRec<V> dest, Curseur<V> c) {
    if (c == null) {
      return;
    }
    V e = source.element(c);
    c = source.suivant(c);
    dupliquerLoc(source, dest, c);
    dest.ajouterDebut(e);
  }

  public boolean equals(ListeRec<T> autre) {
    Curseur<T> c1 = debut();
    Curseur<T> c2 = autre.debut();
    while (c1 != null && c2 != null) {
      if (element(c1) != autre.element(c2)) {
        return false;
      }
      c1 = suivant(c1);
      c2 = autre.suivant(c2);
    }
    return (c1 == null) && (c2 == null);
  }
  protected abstract ListeRec<T> makeInstance();
/*P-*/
}
/*-*/