/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "ListeRecChainee.java"
 *                                    created: 2003-02-10 12:29:49
 *                                last update: 2008-02-27 10:04:15
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *  Implmentation par liste chane de la liste rcursive.
 *
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2003-02-10 FBO 1.0 original
 * ###################################################################
 */
package collections;


/**
 * Liste rcursive dont les lments sont rangs dans une liste chane.
 */
/*+ListeRecChainee/base.java*/
public class ListeRecChainee<T> extends ListeRec<T> {
  /** Maillon portant le premier lment de la liste. */
  private Maillon premier_;

  /** Nombre d'lments dans la liste. */
  private int longueur_;

  // Oprations de la spcification.

  /** Constructeur = listevide. */
  public ListeRecChainee() {
    premier_ = null;
    longueur_ = 0;
  }

  public void ajouterDebut(T e) {
    premier_ = new Maillon(e, premier_);
    longueur_++;
  }

  public void enleverPremier() {
    prerequis(longueur_ > 0);
    premier_ = premier_.suivant;
    longueur_--;
  }

  public T premier() {
    prerequis(longueur_ > 0);
    return premier_.element;
  }

  public boolean estvide() {
    return longueur_ == 0;
  }

  public int longueur() {
    return longueur_;
  }
/*-*/
  /**
   * Implmente la notion de curseur : dans une liste reprsente par
   * liste chane, un curseur est simplement un maillon de la liste.
   */
/*+ListeRecChainee/classeMaillon.java*/
  private class Maillon extends Curseur<T> {
    /**
     * On ne protge pas les membres de cette classe. En effet,
     * <code>Maillon</code> est une classe prive qui ne peut tre
     * vue par aucune autre classe que <code>ListeIterChainee</code>.
     * De plus, son comportement est intimement li  celui de la liste.
     * Il est donc inutile de diminuer l'efficacit du code par des
     * appels de mthodes qui n'apportent rien au niveau de l'encapsulation
     * ou du polymorphisme.
     * Notez qu'il s'agit l d'une situation exceptionnelle.
     */
    public T element;
    public Maillon suivant;

    public Maillon(T e, Maillon s) {
      element = e;
      suivant = s;
    }
  }
/*-*/
  // Oprations sur les curseurs.
  //
  // Les oprations qui reoivent un Curseur devraient vrifier que
  // ce curseur appartient bien  la liste. Ici, cette vrification
  // n'est pas faite, et utiliser un curseur invalide provoquera donc
  // un comportement erratique sans qu'une erreur soit forcment signale.
  // Une solution consisterait  placer une rfrence sur la liste dans
  // laquelle le Curseur a un sens et  vrifier que cette liste est
  // la bonne  chaque appel. Cela viterait les comportements erratiques
  // silencieux mais dgraderait les performances.

  public Curseur<T> debut() {
    return premier_;
  }

  public Curseur<T> suivant(Curseur<T> c) {
    Maillon m = (Maillon)c;
    return m.suivant;
  }

  public T element(Curseur<T> c) {
    Maillon m = (Maillon)c;
    return m.element;
  }

  public void remplacer(Curseur<T> c, T e) {
    Maillon m = (Maillon)c;
    m.element = e;
  }

  public void supprimerSuivant(Curseur<T> c) {
    Maillon m = (Maillon)c;
    if (m.suivant == null) {
      throw new Error("Curseur invalide pour supprimerSuivant()");
    }
    m.suivant = m.suivant.suivant;
  }

  public void insererApres(Curseur<T> c, T e) {
    Maillon m = (Maillon)c;
    if ( m == null )
      premier_ = new Maillon(e, premier_);
    else
      m.suivant = new Maillon(e, m.suivant);
  }

  protected ListeRecChainee<T> makeInstance() {
    return new ListeRecChainee<T>();
  }
  
  // Mthode de test
  public static void main(String[] args) {
    ListeRec<Object> l = new ListeRecChainee<Object>();
    l.ajouterDebut("d");
    ListeRec<String> l2 = new ListeRecChainee<String>();
    l2.ajouterDebut("c");
    l2.ajouterDebut("b");
    l.ajouterDebut(l2);
    l.ajouterDebut("a");
    System.out.println(l.toString());
    l2.ajouterDebut("b'");
    l.ajouterDebut("a'");
    System.out.println(l.toString());
    l2.enleverPremier();
    l.enleverPremier();
    System.out.println(l.toString());
    Curseur<Object> c = l.debut();
    l.remplacer(c,"A");
    Curseur<String> c2 = l2.debut();
    l2.remplacer(c2,"B");
    System.out.println(l.toString());
    l.insererApres(c,"A'");
    System.out.println(l.toString());
    // Provoquer une erreur car c n'est pas un curseur du bon type pour l2!
    // l2.insererApres(c,"Z");
    System.out.println(l.toString());
  }
}
