/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "Parcours.java"
 *                                    created: 2002-10-03 15:55:02
 *                                last update: 2008-02-22 17:00:32
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *
 *  Diffrents parcours d'arbres binaires.
 *
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2002-10-03 FBO 1.0 original
 * ###################################################################
 */
package collections;
import java.util.LinkedList;

/**
 * Permet de parcourir un arbre binaire en pr-ordre, en ordre,
 * en post-ordre et en largeur, en appliquant un traitement  chaque
 * noeud parcouru.
 */
public class Parcours {
  /** Mthode de test. */
  public static void main(String[] args) {
    Parcours parcours1 = new Parcours();
    Traitement<Object> affiche = new Traitement<Object>() {
      public void traiter(Object o) {
        System.out.print(o + " ");
      }
    };
    Arbre<String> a = new Arbre<String>(new Arbre<String>(),"1", new Arbre<String>());
    Curseur<String> c = a.curseurRacine();
    a.ajouterFilsGauche("2",c);
    a.ajouterFilsDroit("3",c);
    c = a.versFilsGauche(c);
    a.ajouterFilsGauche("4",c);
    a.ajouterFilsDroit("5",c);
    c = a.curseurRacine();
    c = a.versFilsDroit(c);
    a.ajouterFilsGauche("6",c);
    a.ajouterFilsDroit("7",c);
    c = a.versFilsDroit(c);
    a.ajouterFilsGauche("8",c);
    a.ajouterFilsDroit("9",c);
    System.out.println("Arbre :      1       ");
    System.out.println("            / \\      ");
    System.out.println("           /   \\     ");
    System.out.println("          2     3    ");
    System.out.println("         / \\   / \\   ");
    System.out.println("        4   5 6   7  ");
    System.out.println("                 / \\ ");
    System.out.println("                8   9");
    System.out.print("Parcours en prordre : ");
    preOrdre(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en ordre : ");
    enOrdre(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en post-ordre : ");
    postOrdre(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en largeur : ");
    largeur(a, a.curseurRacine(), affiche);
    System.out.println();
  }

  /**
   * Parcourt l'arbre <code>a</code> en pr-ordre  partir du
   * noeud dsign par <code>c</code>, et applique le traitement
   * <code>t</code>  chacun des noeuds parcourus.
   */
  static public <T> void preOrdre(Arbre<T> a, Curseur<T> c, Traitement<? super T> t) {
    // Traiter le noeud, puis ses fils (notation pr-fixee).
    if (c != null) {
      t.traiter(a.element(c));
      preOrdre(a, a.versFilsGauche(c), t);
      preOrdre(a, a.versFilsDroit(c), t);
    }
  }

  /**
   * Parcourt l'arbre <code>a</code> en ordre  partir du
   * noeud dsign par <code>c</code>, et applique le traitement
   * <code>t</code>  chacun des noeuds parcourus.
   */
  static public <T> void enOrdre(Arbre<T> a, Curseur<T> c, Traitement<? super T> t) {
    // Traiter le fils gauche, le noeud, et fils droit (notation infixe)
    if (c != null) {
      enOrdre(a, a.versFilsGauche(c), t);
      t.traiter(a.element(c));
      enOrdre(a, a.versFilsDroit(c), t);
    }
  }

  /**
   * Parcourt l'arbre <code>a</code> en largeur  partir du
   * noeud dsign par <code>c</code>, et applique le traitement
   * <code>t</code>  chacun des noeuds parcourus.
   */
  static public <T> void largeur(Arbre<T> a, Curseur<T> c, Traitement<? super T> t) {
    // De la racine vers les feuilles et de la gauche vers la droite.

    File<Curseur<T>> f = new FileLinkedList<Curseur<T>>(); // Les noeuds  parcourir sont rangs dans une file
    Curseur<T> courant;

    if (c != null) {
      f.ajouter(c);      // Le premier noeud est plac dans la file
    }

    while (!f.estvide()) {
      // Tant que la file n'est pas vide, on traite son premier lment
      courant = f.premier();
      f.enlever();
      t.traiter(a.element(courant));
      // et on ajoute ses ventuels fils  la file
      if (a.versFilsGauche(courant) != null) {
        f.ajouter(a.versFilsGauche(courant));
      }
      if (a.versFilsDroit(courant) != null) {
        f.ajouter(a.versFilsDroit(courant));
      }
    }
  }

  /**
   * Parcourt l'arbre <code>a</code> en post-ordre  partir du
   * noeud dsign par <code>c</code>, et applique le traitement
   * <code>t</code>  chacun des noeuds parcourus.
   */
  static public <T> void postOrdre(Arbre<T> a, Curseur<T> c, Traitement<? super T> t) {
    // Traiter d'abord les fils, puis le noeud (notation post-fixe).
    if (c != null) {
      postOrdre(a, a.versFilsGauche(c), t);
      postOrdre(a, a.versFilsDroit(c), t);
      t.traiter(a.element(c));
    }
  }
}

