/* -*-Java-*-
 * ###################################################################
 *
 *  FILE: "Parcours.java"
 *                                    created: 2002-09-24 17:27:12
 *                                last update: 2008-02-22 17:00:16
 *  Author: Frdric Boulanger
 *  E-mail: Frederic.Boulanger@supelec.fr
 *    mail: Suplec - Service Informatique
 *          3 rue Joliot-Curie, F-91192 Gif-sur-Yvette cedex
 *     www: http://wwwsi.supelec.fr/fb/
 *
 *  Description:
 *
 *  Diffrents parcours de fort (ou d'arbres planaires)
 *  History
 *
 *  modified   by  rev reason
 *  ---------- --- --- -----------
 *  2002-09-24 FBO 1.0 original
 * ###################################################################
 */
package collections;
import java.util.LinkedList;


/**
 * Permet de parcourir une fort en pr-ordre, en prdre, en post-ordre
 * et en largeur, en appliquant un traitement  chaque noeud parcouru.
 */
public class ParcoursForet {
  /* Mthode de test des parcours */
  public static void main(String[] args) {
    // On cre ici une instance d'une classe anonyme qui implmente
    // l'interface traitement. La dfinition de cette classe suit
    // immdiatement le "new Traitement()". Ici, le Traitement affiche
    // simplement l'objet, suivi d'un espace, sur System.out.
    Traitement<Object> affiche = new Traitement<Object>() {
      public void traiter(Object o) {
        System.out.print(o + " ");
      }
    };
    Foret<String> a = new Foret<String>(new Foret<String>(),"1", new Foret<String>());
    Curseur<String> c = a.curseurRacine();
    a.ajouterFilsAine("4",c);
    a.ajouterFilsAine("3",c);
    a.ajouterFilsAine("2",c);
    c = a.versFilsAine(c);
    a.ajouterFilsAine("6",c);
    a.ajouterFilsAine("5",c);
    c = a.versFrereCadet(c);
    a.ajouterFilsAine("9",c);
    a.ajouterFilsAine("8",c);
    a.ajouterFilsAine("7",c);
    c = a.versFilsAine(c);
    c = a.versFrereCadet(c);
    a.ajouterFilsAine("11",c);
    a.ajouterFilsAine("10",c);
    System.out.println("Fort :        1       ");
    System.out.println("             / | \\      ");
    System.out.println("            /  |  \\      ");
    System.out.println("           /   |   \\     ");
    System.out.println("          /    |    \\     ");
    System.out.println("         2     3     4    ");
    System.out.println("        / \\   /|\\   ");
    System.out.println("       5   6 7 8 9  ");
    System.out.println("              / \\ ");
    System.out.println("             10  11");
    System.out.print("Parcours en pr-ordre : ");
    preordreRec(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en pr-ordre (autre mthode) : ");
    preordreIter(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en ordre : ");
    enordre(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en post-ordre : ");
    postordreRec(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en post-ordre (autre mthode) : ");
    postordreIter(a, a.curseurRacine(), affiche);
    System.out.println();
    System.out.print("Parcours en largeur : ");
    largeur(a, a.curseurRacine(), affiche);
    System.out.println();
  }

  /**
   * Parcourt la fort <code>f</code> en pr-ordre  partir
   * du noeud indiqu par <code>c</code>, et applique le
   * traitement <code>t</code>  chacun des noeuds parcourus.
   * Cette version est similaire au parcours en pr-ordre
   * d'un arbre binaire, le fils ain jouant le rle de fils
   * gauche, et le frre cadet celui de fils droit.
   */
/*+ParcoursForet/preordreRec.java*/
  static public <T> void preordreRec(Foret<T> f, Curseur<T> c,
                                     Traitement<? super T> t) {
    if (c != null) {
      // on commence par traiter l'lment
      t.traiter(f.element(c));
      // puis on parcourt la fort fille en pr-ordre
      preordreRec(f, f.versFilsAine(c), t);
      // et on parcourt la fort soeur en pr-ordre
      preordreRec(f, f.versFrereCadet(c), t);
    }
  }
/*-*/

  /**
   * Parcourt la fort <code>f</code> en pr-ordre  partir
   * du noeud indiqu par <code>c</code>, et applique le
   * traitement <code>t</code>  chacun des noeuds parcourus.
   * Cette version traite l'lment, puis tous ses fils en partant
   * du fils an.
   */
/*+ParcoursForet/preordreIter.java*/
  static public <T> void preordreIter(Foret<T> f, Curseur<T> c,
                                      Traitement<? super T> t) {
    if (c != null) {
      // on commence par traiter l'lment
      t.traiter(f.element(c));
      // puis on parcourt tous les fils en partant de l'an
      c = f.versFilsAine(c);
      while (c != null) {
        preordreIter(f, c, t);
        c = f.versFrereCadet(c);
      }
    }
  }
/*-*/

  /**
   * Parcourt la fort <code>f</code> en ordre  partir
   * du noeud indiqu par <code>c</code>, et applique le
   * traitement <code>t</code>  chacun des noeuds parcourus.
   * La notion de parcours en ordre est tendue aux fort  partir
   * de celle dfinie pour les arbres binaires en traitant d'abord le
   * fils an, puis l'lment, puis tous les frres du fils an.
   */
/*+ParcoursForet/ordre.java*/
  static public <T> void enordre(Foret<T> f, Curseur<T> c,
                                 Traitement<? super T> t) {
    if (c != null) {
      // on commence par le fils an
      enordre(f, f.versFilsAine(c), t);
      // puis on traite l'lment
      t.traiter(f.element(c));
      // et on termine avec les frres du fils an
      // s'il y en a.
      c = f.versFilsAine(c);
      if (c != null) {
        // si le fils an existe, on passe  son cadet
        // puisque l'an a dj t trait.
        c = f.versFrereCadet(c);
      }
      while (c != null) {
        enordre(f, c, t);
        c = f.versFrereCadet(c);
      }
    }
  }
/*-*/
  /**
   * Parcourt la fort <code>f</code> en largeur d'abord  partir
   * du noeud indiqu par <code>c</code>, et applique le
   * traitement <code>t</code>  chacun des noeuds parcourus.
   */
/*+ParcoursForet/largeur.java*/
  static public <T> void largeur(Foret<T> f, Curseur<T> c,
                                 Traitement<? super T> t) {
    File<Curseur<T>> q = new FileChainee<Curseur<T>>();

    if (c != null) {
      q.ajouter(c);
    }
    while (!q.estvide()) {
      // tant que la file n'est pas vide, on traite
      // son premier lment, que l'on retire de la file
      Curseur<T> courant = q.premier();
      q.enlever();
      t.traiter(f.element(courant));
      // puis on ajoute tous ses fils  la file
      courant = f.versFilsAine(courant);
      while(courant != null) {
        q.ajouter(courant);
        courant = f.versFrereCadet(courant);
      }
    }
  }
/*-*/
  /**
   * Parcourt la fort <code>f</code> en post-ordre  partir du noeud
   * indiqu par <code>c</code>, et applique le traitement <code>t</code>
   *  chacun des noeuds parcourus. Cette version est similaire au
   * parcours en ordre d'un arbre binaire (ce qui peut sembler curieux), le
   * fils ain jouant le rle de fils gauche, et le frre cadet celui de
   * fils droit.
   */
/*+ParcoursForet/postordreRec.java*/
  static public <T> void postordreRec(Foret<T> f, Curseur<T> c,
                                      Traitement<? super T> t) {
    if (c != null) {
      // le parcours en post-ordre  partir du fils an parcourt
      // toute la fort fille du noeud
      postordreRec(f, f.versFilsAine(c), t);
      // comme tous les fils ont t traits, il faut maintenant
      // traiter l'lment
      t.traiter(f.element(c));
      // avant de parcourir la fort soeur.
      postordreRec(f, f.versFrereCadet(c), t);
    }
  }
/*-*/
  /**
   * Parcourt la fort <code>f</code> en post-ordre  partir du noeud
   * indiqu par <code>c</code>, et applique le traitement <code>t</code>
   *  chacun des noeuds parcourus. Cette version traite tous les fils en
   * partant du fils an, puis l'lment, ce qui correspond
   * littralement  la dfinition du post-ordre.
   */
/*+ParcoursForet/postordreIter.java*/
  static public <T> void postordreIter(Foret<T> f, Curseur<T> c,
                                       Traitement<? super T> t) {
    if (c != null) {
      // partant du fils an, on parcourt tous les fils
      // en post-ordre
      Curseur<T> courant = f.versFilsAine(c);
      while (courant != null) {
        postordreIter(f, courant, t);
        courant = f.versFrereCadet(courant);
      }
      // puis on traite l'lment en dernier.
      t.traiter(f.element(c));
    }
  }
/*-*/
}

